#!/usr/bin/python
#
# test-sysctlconfig.py
#
#     Use known sysctl.d output to regression test the sysctl.d
#     configuration generation.
#
#     Returns 0 when the output looks good.
#
#     Returns -1 when the output differs from expected along with a diff of the
#     unexpected change.
#
#     When a platform is added to platforms.all_platforms, this test will
#     initially fail and complain about a missing test_vector directory.
#     Creating the directory "test_vectors/platform/ is enough to silence the
#     failure.  To add a test:
#        1. run test-sysctlconfig.py -g to generate expected configuration output
#
#     Example:
#        PYTHONPATH=../../dist-packages/ ./test-sysctlconfig.py

import os
import time
import sys
import argparse
import difflib
import subprocess
import re

import cumulus.sysctlconfig
import cumulus.platforms

def get_vector_path():
    base_path = os.path.dirname(os.path.abspath(__file__))
    vector_path = os.path.join(base_path, 'test_vectors')
    return vector_path

def get_ledasm_path(ledasm_path):
    if os.path.exists(ledasm_path):
        return True
    else:
        return False

def gen_vectors():
    vector_path = get_vector_path()
    for platform_name in os.listdir(vector_path):
        try:
            test_dirs = os.listdir(os.path.join(vector_path, platform_name))
        except OSError:
            print 'warning: missing test directories for: %s' % platform_name
            continue

        for test_dir in test_dirs:
            platform = cumulus.platforms.probe(forced_platform=platform_name)
            sc = cumulus.sysctlconfig.Config(platform)

            for sysctl in ['neigh', 'igmp', 'mld']:

                out = open(os.path.join(vector_path,
                                        platform_name,
                                        test_dir,
                                        sysctl), 'w')
                out.write(sc.output(sysctl))

def read_vectors(platform=None):
    # Read the test_vectors directory and create a list of tests to run.
    test_vectors = []
    vector_path = get_vector_path()

    if platform is None:
        platform_paths = os.listdir(vector_path)
    else:
        platform_paths = [platform]

    for platform_dir in platform_paths:
        platform = platform_dir

        vec = {}
        vec['platform'] = cumulus.platforms.probe()
        for config_file in os.listdir(os.path.join(vector_path, platform_dir)):
            vec[config_file] = open(os.path.join(vector_path, platform_dir, config_file), 'r').read()

        test_vectors.append(vec)

    return test_vectors

def compare(generated, expected):
    if generated != expected:
        return '\n'.join(difflib.unified_diff(expected.split('\n'),
                                              generated.split('\n'),
                                              fromfile='expected',
                                              tofile='actual',
                                              n=999999,
                                              lineterm=''))

def do_test(test_vectors):
    failed = False

    for vec in test_vectors:
        sc = cumulus.sysctlconfig.Config(vec['platform'].name)

        for sysctl in ['neigh', 'igmp', 'mld']:
            diff = compare(sc.output(sysctl), vec[sysctl])
            if diff is not None:
                 failed = True
                 print "Test Vector FAILED: platform=%s file=%s" % \
                       (vec['platform'], sysctl)
                 print diff

    if failed:
        sys.exit(-1)
    else:
        print "success"
        sys.exit(0)

########################################################################
#
# MAIN
#
########################################################################
if __name__ == '__main__':
    parser = argparse.ArgumentParser(
        description='test cumulus port configuration modules')
    parser.add_argument('-g', '--generate-vectors',
                        required=False,
                        default=False,
                        action='store_true',
                        help='Generate test vectors to output directory')
    parser.add_argument('-d', '--vector-dir',
                        required=False,
                        default=None,
                        action='store',
                        help='Use specified test vector directory')
    args = parser.parse_args()

    if args.generate_vectors is False:
        vectors = read_vectors(args.vector_dir)
        do_test(vectors)
    else:
        print 'Generating new test vectors'
        gen_vectors()
